@@ -116,23 +116,19 @@ each_contact (MuMsgContact *c, ContactData *cdata)
 	case MU_MSG_CONTACT_TYPE_FROM:
 		add_prefix_maybe (cdata->gstr, &cdata->from, "\t:from (");
 		break;
-
 	case MU_MSG_CONTACT_TYPE_TO:
 		add_prefix_maybe (cdata->gstr, &cdata->to, "\t:to (");
 		break;
-
 	case MU_MSG_CONTACT_TYPE_CC:
 		add_prefix_maybe (cdata->gstr, &cdata->cc, "\t:cc (");
 		break;
-
 	case MU_MSG_CONTACT_TYPE_BCC:
 		add_prefix_maybe (cdata->gstr, &cdata->bcc, "\t:bcc (");
 		break;
-
 	case MU_MSG_CONTACT_TYPE_REPLY_TO:
-		add_prefix_maybe (cdata->gstr, &cdata->reply_to, "\t:reply-to (");
+		add_prefix_maybe (cdata->gstr, &cdata->reply_to,
+				  "\t:reply-to (");
 		break;
-
 	default: g_return_val_if_reached (FALSE);
 	}
 
@@ -202,77 +198,137 @@ append_sexp_flags (GString *gstr, MuMsg *msg)
 }
 
 static char*
-get_temp_file (MuMsg *msg, unsigned index)
+get_temp_file (MuMsg *msg, MuMsgOptions opts, unsigned index)
 {
 	char *path;
 	GError *err;
 
 	err = NULL;
-	path = mu_msg_part_filepath_cache (msg, index);
-	if (!mu_msg_part_save (msg, path, index,
-			       FALSE/*overwrite*/, TRUE/*use cache*/, &err)) {
-		g_warning ("failed to save mime part: %s",
-			   err->message ? err->message : "something went wrong");
-		g_clear_error (&err);
-		g_free (path);
-		return NULL;
-	}
+	path = mu_msg_part_get_cache_path (msg, opts, index, &err);
+	if (!path)
+		goto errexit;
+
+	if (!mu_msg_part_save (msg, opts, path, index, &err))
+		goto errexit;
 
 	return path;
+
+errexit:
+	g_warning ("failed to save mime part: %s",
+		   err->message ? err->message : "something went wrong");
+	g_clear_error (&err);
+	g_free (path);
+	return NULL;
+}
+
+
+static gchar*
+get_temp_file_maybe (MuMsg *msg, MuMsgPart *part, MuMsgOptions opts)
+{
+	char *tmp, *tmpfile;
+
+	opts |= MU_MSG_OPTION_USE_EXISTING;
+
+	if  (!(opts & MU_MSG_OPTION_EXTRACT_IMAGES) ||
+	     g_ascii_strcasecmp (part->type, "image") != 0)
+		return NULL;
+
+	tmp = get_temp_file (msg, opts, part->index);
+	if (!tmp)
+		return NULL;
+
+	tmpfile = mu_str_escape_c_literal (tmp, TRUE);
+	g_free (tmp);
+	return tmpfile;
 }
 
 
 struct _PartInfo {
-	char *parts;
-	gboolean want_images;
+	char        *parts;
+	MuMsgOptions opts;
 };
 typedef struct _PartInfo PartInfo;
 
-
-/* like the elvis operator,
- * http://colinharrington.net/blog/2008/10/groovy-elvis-operator/
- */
 static const char*
-elvis (const char *s1, const char *s2)
+sig_verdict (MuMsgPart *mpart)
 {
-	return s1 ? s1 : s2;
+	MuMsgPartSigStatusReport *report;
+
+	report = mpart->sig_status_report;
+	if (!report)
+		return "";
+
+	switch (report->verdict) {
+	case MU_MSG_PART_SIG_STATUS_GOOD:
+		return ":signature verified";
+	case MU_MSG_PART_SIG_STATUS_BAD:
+		return ":signature bad";
+	case MU_MSG_PART_SIG_STATUS_ERROR:
+		return ":signature unverified";
+	default:
+		return "";
+	}
 }
 
+
+static gchar *
+get_part_type_string (MuMsgPartType ptype)
+{
+	GString *gstr;
+	unsigned u;
+	struct PartTypes {
+		MuMsgPartType ptype;
+		const char* name;
+	} ptypes[] = {
+		{ MU_MSG_PART_TYPE_LEAF,       "leaf" },
+		{ MU_MSG_PART_TYPE_MESSAGE,    "message" },
+		{ MU_MSG_PART_TYPE_INLINE,     "inline" },
+		{ MU_MSG_PART_TYPE_ATTACHMENT, "attachment" },
+		{ MU_MSG_PART_TYPE_SIGNED,     "signed" },
+		{ MU_MSG_PART_TYPE_ENCRYPTED,  "encrypted" }
+	};
+
+	gstr = g_string_sized_new (100); /* more than enough */
+	gstr = g_string_append_c (gstr, '(');
+
+	for (u = 0; u!= G_N_ELEMENTS(ptypes); ++u) {
+		if (ptype & ptypes[u].ptype) {
+			if (gstr->len > 1)
+				gstr = g_string_append_c (gstr, ' ');
+			gstr = g_string_append (gstr, ptypes[u].name);
+		}
+	}
+
+	gstr = g_string_append_c (gstr, ')');
+
+	return g_string_free (gstr, FALSE);
+}
+
+
 static void
 each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 {
-	const char *fname;
-	char *name, *tmp;
+	char *name, *tmp, *parttype;
 	char *tmpfile;
 
-	if (!(fname = mu_msg_part_file_name (part)))
-		fname = mu_msg_part_description (part);
-	if (fname)
-		name = mu_str_escape_c_literal (fname, TRUE);
-	else
-		name = g_strdup_printf ("\"%s-%s-%d\"",
-					elvis (part->type, "application"),
-					elvis (part->subtype, "octet-stream"),
-					part->index);
-
-	tmpfile = NULL;
-	if (pinfo->want_images && g_ascii_strcasecmp (part->type, "image") == 0) {
-		char *tmp;
-		if ((tmp = get_temp_file (msg, part->index))) {
-			tmpfile = mu_str_escape_c_literal (tmp, TRUE);
-			g_free (tmp);
-		}
-	}
+	name     = mu_msg_part_get_filename (part, TRUE);
+	tmpfile  = get_temp_file_maybe (msg, part, pinfo->opts);
+	parttype = get_part_type_string (part->part_type);
 
 	tmp = g_strdup_printf
-		("%s(:index %d :name %s :mime-type \"%s/%s\"%s%s "
-		 ":attachment %s :size %i)",
-		 elvis (pinfo->parts, ""), part->index, name,
-		 elvis (part->type, "application"),
-		 elvis (part->subtype, "octet-stream"),
+		("%s(:index %d :name \"%s\" :mime-type \"%s/%s\"%s%s "
+		 ":type %s "
+		 ":attachment %s :size %i %s)",
+		 pinfo->parts ? pinfo->parts: "",
+		 part->index,
+		 name ? name : "noname",
+		 part->type ? part->type : "application",
+		 part->subtype ? part->subtype : "octet-stream",
 		 tmpfile ? " :temp" : "", tmpfile ? tmpfile : "",
-		 mu_msg_part_looks_like_attachment (part, TRUE) ? "t" : "nil",
-		 (int)part->size);
+		 parttype,
+		 mu_msg_part_maybe_attachment (part) ? "t" : "nil",
+		 (int)part->size,
+		 sig_verdict (part));
 
 	g_free (pinfo->parts);
 	pinfo->parts = tmp;
@@ -280,36 +336,23 @@ each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 
 
 static void
-append_sexp_parts (GString *gstr, MuMsg *msg, gboolean want_images)
+append_sexp_parts (GString *gstr, MuMsg *msg, MuMsgOptions opts)
 {
 	PartInfo pinfo;
 
-	pinfo.parts       = NULL;
-	pinfo.want_images = want_images;
+	pinfo.parts = NULL;
+	pinfo.opts  = opts;
 
-	mu_msg_part_foreach (msg, FALSE,
-			     (MuMsgPartForeachFunc)each_part, &pinfo);
-
-	if (pinfo.parts) {
+	if (!mu_msg_part_foreach (msg, opts, (MuMsgPartForeachFunc)each_part,
+				  &pinfo)) {
+		/* if decryption failed, mark this message as encrypted */
+		g_string_append (gstr, "\t:encrypted t\n");
+	} else if (pinfo.parts) {
 		g_string_append_printf (gstr, "\t:parts (%s)\n", pinfo.parts);
 		g_free (pinfo.parts);
 	}
 }
 
-
-
-static void
-append_sexp_message_file_attr (GString *gstr, MuMsg *msg)
-{
- 	append_sexp_attr_list (gstr, "references", mu_msg_get_references (msg));
-	append_sexp_attr (gstr, "in-reply-to",
-			  mu_msg_get_header (msg, "In-Reply-To"));
-	append_sexp_attr (gstr, "body-txt",
-			  mu_msg_get_body_text(msg));
-	append_sexp_attr (gstr, "body-html",
-			  mu_msg_get_body_html(msg));
-}
-
 static void
 append_sexp_thread_info (GString *gstr, const MuMsgIterThreadInfo *ti)
 {
@@ -328,53 +371,97 @@ append_sexp_thread_info (GString *gstr, const MuMsgIterThreadInfo *ti)
 }
 
 
+static void
+append_message_file_parts (GString *gstr, MuMsg *msg, MuMsgOptions opts)
+{
+	GError *err;
+	err = NULL;
+
+	if (!mu_msg_load_msg_file (msg, &err)) {
+		g_warning ("failed to load message file: %s",
+			   err ? err->message : "some error occured");
+		g_clear_error (&err);
+		return;
+	}
+
+	append_sexp_parts (gstr, msg, opts);
+	append_sexp_contacts (gstr, msg);
+
+	append_sexp_attr_list (gstr, "references",
+			       mu_msg_get_references (msg));
+	append_sexp_attr (gstr, "in-reply-to",
+			  mu_msg_get_header (msg, "In-Reply-To"));
+	append_sexp_attr (gstr, "body-txt",
+			  mu_msg_get_body_text(msg, opts));
+	append_sexp_attr (gstr, "body-html",
+			  mu_msg_get_body_html(msg, opts));
+}
+
+static void
+append_sexp_date_and_size (GString *gstr, MuMsg *msg)
+{
+	time_t t;
+	size_t s;
+
+	t = mu_msg_get_date (msg);
+	if (t == (time_t)-1)  /* invalid date? */
+		t = 0;
+
+	s = mu_msg_get_size (msg);
+	if (s == (size_t)-1)   /* invalid size? */
+		s = 0;
+
+	g_string_append_printf
+		(gstr,
+		 "\t:date (%u %u 0)\n\t:size %u\n",
+		 (unsigned)(t >> 16),
+		 (unsigned)(t & 0xffff),
+		 (unsigned)s);
+}
+
+
 char*
 mu_msg_to_sexp (MuMsg *msg, unsigned docid, const MuMsgIterThreadInfo *ti,
-		gboolean header_only, gboolean extract_images)
+		MuMsgOptions opts)
 {
 	GString *gstr;
-	time_t t;
 
 	g_return_val_if_fail (msg, NULL);
-	g_return_val_if_fail (!(header_only && extract_images), NULL);
+	g_return_val_if_fail (!((opts & MU_MSG_OPTION_HEADERS_ONLY) &&
+				(opts & MU_MSG_OPTION_EXTRACT_IMAGES)),NULL);
+	gstr = g_string_sized_new
+		((opts & MU_MSG_OPTION_HEADERS_ONLY) ?  1024 : 8192);
 
-	gstr = g_string_sized_new (header_only ? 1024 : 8192);
-	g_string_append (gstr, "(\n");
-
-	if (docid != 0)
-		g_string_append_printf (gstr, "\t:docid %u\n", docid);
+	if (docid == 0)
+		g_string_append (gstr, "(\n");
+	else
+		g_string_append_printf (gstr, "(\n\t:docid %u\n", docid);
 
 	if (ti)
 		append_sexp_thread_info (gstr, ti);
 
 	append_sexp_attr (gstr, "subject", mu_msg_get_subject (msg));
 
-	t = mu_msg_get_date (msg);
-	/* weird time format for emacs 29-bit ints...*/
-	g_string_append_printf (gstr,"\t:date (%u %u 0)\n", (unsigned)(t >> 16),
-				(unsigned)(t & 0xffff));
-	g_string_append_printf (gstr, "\t:size %u\n",
-				(unsigned) mu_msg_get_size (msg));
+	/* in the no-headers-only case (see below) we get a more
+	 * complete list of contacts, so no need to get them here if
+	 * that's the case */
+	if (opts & MU_MSG_OPTION_HEADERS_ONLY)
+		append_sexp_contacts (gstr, msg);
+
+	append_sexp_date_and_size (gstr, msg);
+
 	append_sexp_attr (gstr, "message-id", mu_msg_get_msgid (msg));
 	append_sexp_attr (gstr, "path",	 mu_msg_get_path (msg));
 	append_sexp_attr (gstr, "maildir", mu_msg_get_maildir (msg));
-
 	g_string_append_printf (gstr, "\t:priority %s\n",
 				mu_msg_prio_name(mu_msg_get_prio(msg)));
-
 	append_sexp_flags (gstr, msg);
 
-	/* headers are retrieved from the database, views from the message file
-	 * file attr things can only be gotten from the file (ie., mu
-	 * view), not from the database (mu find).  */
-	if (!header_only) {
-		append_sexp_message_file_attr (gstr, msg);
-		append_sexp_parts (gstr, msg, extract_images);
-	}
-
-	/* note, some of the contacts info comes from the file, soe
-	 * this has to be after the previous */
-	append_sexp_contacts (gstr, msg);
+	/* headers are retrieved from the database, views from the
+	 * message file file attr things can only be gotten from the
+	 * file (ie., mu view), not from the database (mu find).  */
+	if (!(opts & MU_MSG_OPTION_HEADERS_ONLY))
+		append_message_file_parts (gstr, msg, opts);
 
 	g_string_append (gstr, ")\n");
 	return g_string_free (gstr, FALSE);
